//----------------------------------------------------------------------//
// GitrA[															//
//																		//
//----------------------------------------------------------------------//
#include <vcl.h>
#pragma hdrstop
#include <memory>
#include <algorithm>
#include <System.StrUtils.hpp>
#include "UserFunc.h"
#include "UserMdl.h"
#include "Global.h"
#include "GitView.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TGitViewer *GitViewer = NULL;

//---------------------------------------------------------------------------
__fastcall TGitViewer::TGitViewer(TComponent* Owner)
	: TForm(Owner)
{
	HistoryLimit = GIT_DEF_HISTLIMIT;
}
//---------------------------------------------------------------------------
void __fastcall TGitViewer::FormCreate(TObject *Sender)
{
	CommitList  = new TStringList();

	BranchScrPanel = new UsrScrollPanel(BranchPanel, BranchListBox, USCRPNL_FLAG_P_WP | USCRPNL_FLAG_L_WP);
	CommitScrPanel = new UsrScrollPanel(CommitPanel, CommitListBox, USCRPNL_FLAG_P_WP | USCRPNL_FLAG_L_WP);
	DiffScrPanel   = new UsrScrollPanel(DiffPanel,   DiffListBox,   USCRPNL_FLAG_P_WP | USCRPNL_FLAG_L_WP);
}
//---------------------------------------------------------------------------
void __fastcall TGitViewer::FormShow(TObject *Sender)
{
	RepoDir = ExcludeTrailingPathDelimiter(GetCurrentDir());
	Caption = yen_to_delimiter(RepoDir) + " - GitrA[";

	IniFile->LoadPosInfo(this);

	set_ListBoxItemHi(BranchListBox, ListFont);
	set_UsrScrPanel(BranchScrPanel);
	set_ListBoxItemHi(CommitListBox, ListFont);
	set_UsrScrPanel(CommitScrPanel);
	set_ListBoxItemHi(DiffListBox,	 TxtPrvFont);
	set_UsrScrPanel(DiffScrPanel);

	BranchPanel->Width = IniFile->ReadIntGen(_T("GitViewBranchWidth"),	200);
	DiffPanel->Height  = IniFile->ReadIntGen(_T("GitViewDiffHeight"),	120);
	ShowBranchesAction->Checked = IniFile->ReadBoolGen(_T("GitViewShowBranches"));
	ShowRemoteAction->Checked	= IniFile->ReadBoolGen(_T("GitViewShowRemote"));
	AppFextColorAction->Checked = IniFile->ReadBoolGen(_T("GitViewFExtColor"), true);

	CommitSplitter->Color = col_Splitter;
	DiffSplitter->Color   = col_Splitter;

	BranchListBox->Color  = col_bgList;
	CommitListBox->Color  = col_bgList;
	DiffListBox->Color	  = col_bgTxtPrv;

	::PostMessage(Handle, WM_FORM_SHOWED, 0, 0);
}
//---------------------------------------------------------------------------
void __fastcall TGitViewer::WmFormShowed(TMessage &msg)
{
	Repaint();
	GetCommitList();
	BranchListBox->SetFocus();
}

//---------------------------------------------------------------------------
void __fastcall TGitViewer::FormClose(TObject *Sender, TCloseAction &Action)
{
	BranchListBox->Clear();
	CommitListBox->Clear();
	DiffListBox->Clear();

	HistoryLimit = GIT_DEF_HISTLIMIT;

	IniFile->SavePosInfo(this);
	IniFile->WriteIntGen(_T("GitViewBranchWidth"),		BranchPanel->Width);
	IniFile->WriteIntGen(_T("GitViewDiffHeight"),		DiffPanel->Height);
	IniFile->WriteBoolGen(_T("GitViewShowBranches"),	ShowBranchesAction);
	IniFile->WriteBoolGen(_T("GitViewShowRemote"),		ShowRemoteAction);
	IniFile->WriteBoolGen(_T("GitViewFExtColor"),		AppFextColorAction);
}
//---------------------------------------------------------------------------
void __fastcall TGitViewer::FormDestroy(TObject *Sender)
{
	for (int i=0; i<CommitList->Count; i++) delete (git_rec*)CommitList->Objects[i];
	delete CommitList;

	delete BranchScrPanel;
	delete CommitScrPanel;
	delete DiffScrPanel;
}

//---------------------------------------------------------------------------
void __fastcall TGitViewer::BranchPanelResize(TObject *Sender)
{
	BranchScrPanel->UpdateKnob();
}
//---------------------------------------------------------------------------
void __fastcall TGitViewer::CommitPanelResize(TObject *Sender)
{
	CommitScrPanel->UpdateKnob();
}
//---------------------------------------------------------------------------
void __fastcall TGitViewer::DiffPanelResize(TObject *Sender)
{
	DiffScrPanel->UpdateKnob();
}

//---------------------------------------------------------------------------
UnicodeString __fastcall TGitViewer::GitExeStr(UnicodeString prm)
{
	std::unique_ptr<TStringList> o_lst(new TStringList());
	GitBusy = true;
	DWORD exit_code = 0;
	bool res = GitShellExe(prm, RepoDir, o_lst.get(), &exit_code);
	GitBusy = false;

	UnicodeString ret_str;
	if (res) {
		if (exit_code!=0) {
			msgbox_ERR(o_lst->Text);
		}
		else {
			ret_str = o_lst->Text;
			if (!StartsText("archive", prm)) GetCommitList();
		}
	}
	return ret_str;
}

//---------------------------------------------------------------------------
void __fastcall TGitViewer::ClearCommitList()
{
	for (int i=0; i<CommitList->Count; i++) delete (git_rec*)CommitList->Objects[i];
	CommitList->Clear();
}

//---------------------------------------------------------------------------
void __fastcall TGitViewer::GetCommitList()
{
	TListBox *lp = CommitListBox;
	lp->Clear();
	lp->Items->Add("擾...");
	lp->Repaint();

	GitBusy = true;

	std::unique_ptr<TStringList> o_lst(new TStringList());

	//u`ꗗ
	lp = BranchListBox;
	lp->Clear();
	RefHEAD = EmptyStr;
	if (GitShellExe("branch", RepoDir, o_lst.get())) { 
		std::unique_ptr<TStringList> lst(new TStringList());
		int idx = -1;
		for (int i=0; i<o_lst->Count; i++) {
			UnicodeString lbuf = o_lst->Strings[i];
			bool is_cur = remove_top_s(lbuf, '*');
			lst->AddObject(Trim(lbuf), (TObject *)(NativeInt)(is_cur? 1 : 0));
			if (is_cur) {
				idx = i;
				RefHEAD = lbuf;
			}
		}
		lp->Items->Assign(lst.get());
		lp->ItemIndex = idx;
	}
	BranchScrPanel->UpdateKnob();

	//R~bg
	UnicodeString log_prm = "log --graph";
	if (ShowBranchesAction->Checked) log_prm += " --branches";
	if (HistoryLimit>0) log_prm.cat_sprintf(_T(" -%u"), HistoryLimit);
	log_prm += " --date=format:\"%Y-%m-%d %H:%M:%S\" --pretty=format:\"\t%H\t%ad\t%s\t%d\"";

	if (GitShellExe(log_prm, RepoDir, o_lst.get())) { 
		lp = CommitListBox;
		lp->Clear();
		ClearCommitList();
		int max_glen = 0;
		int idx = -1;
		for (int i=0; i<o_lst->Count; i++) {
			UnicodeString lbuf = o_lst->Strings[i];
			TStringDynArray ibuf = split_strings_tab(lbuf);
			if (ibuf.Length>0) {
				git_rec *gp = new git_rec;
				gp->is_head = false;
				gp->graph	= ibuf[0];
				max_glen	= std::max(max_glen, gp->graph.Length());
				if (ibuf.Length==5) {
					gp->msg  = ibuf[3];
					gp->hash = ibuf[1];

					UnicodeString s = get_in_paren(ibuf[4]);
					TStringDynArray b_buf = SplitString(s, ",");
					for (int j=0; j<b_buf.Length; j++) {
						UnicodeString ss = Trim(b_buf[j]);
						if (remove_top_text(ss, "tag: ")) {
							gp->tag = ss;
						}
						else if (ss.Pos("/")) {
							if (!gp->branch_r.IsEmpty()) gp->branch_r += ",";
							gp->branch_r += ss;
						}
						else {
							gp->is_head = remove_top_text(ss, "HEAD -> ");
							gp->branch	= ss;
							if (gp->is_head) idx = i;
						}
					}

					try {
						gp->f_time = VarToDateTime(ibuf[2]);
					}
					catch (...) {
						gp->f_time = 0;
					}
				}
				CommitList->AddObject(gp->hash, (TObject *)gp);
			}
		}

		lp->Tag = (max_glen + 1) * lp->ItemHeight / 3;
		lp->Items->Assign(CommitList);
		if (idx!=-1) {
			lp->ItemIndex = idx;
			CommitListBoxClick(NULL);
		}
	}
	CommitScrPanel->UpdateKnob();

	GitBusy = false;
}

//---------------------------------------------------------------------------
//u`ꗗ̕`
//---------------------------------------------------------------------------
void __fastcall TGitViewer::BranchListBoxDrawItem(TWinControl *Control, int Index, TRect &Rect,
	TOwnerDrawState State)
{
	TListBox *lp = (TListBox*)Control;
	TCanvas  *cv = lp->Canvas;
	cv->Brush->Color = State.Contains(odSelected)? (State.Contains(odFocused)? col_selItem : col_oppItem) : lp->Color;
	cv->FillRect(Rect);

	int xp = Rect.Left + 4;
	int yp = Rect.Top + 2;

	UnicodeString s  = _T("\u25b6");
	if ((int)lp->Items->Objects[Index]==1) {
		cv->Font->Color = col_GitHEAD;
		cv->TextOut(xp, yp, s);
	}
	xp += cv->TextWidth(s);

	cv->Font->Color = is_SelFgCol(State)? col_fgSelItem : col_fgList;
	cv->TextOut(xp, yp, lp->Items->Strings[Index]);
}
//---------------------------------------------------------------------------
void __fastcall TGitViewer::BranchListBoxDblClick(TObject *Sender)
{
	ChckoutAction->Execute();
}
//---------------------------------------------------------------------------
void __fastcall TGitViewer::BranchListBoxKeyDown(TObject *Sender, WORD &Key, TShiftState Shift)
{
	TListBox *lp = (TListBox*)Sender;
	UnicodeString KeyStr  = get_KeyStr(Key, Shift);
	UnicodeString cmd_F = Key_to_CmdF(KeyStr);

	if		(ExeCmdListBox(lp, cmd_F) || ExeCmdListBox(lp, Key_to_CmdV(KeyStr)))
		;
	else if (USAME_TI(cmd_F, "ReturnList"))			ModalResult = mrCancel;
	else if (is_ToRightOpe(KeyStr, cmd_F))			CommitListBox->SetFocus();
	else if (contained_wd_i(KeysStr_Popup, KeyStr))	show_PopupMenu(lp);
	else if (equal_ENTER(KeyStr)) 					ChckoutAction->Execute();
	else return;

	if (!is_DialogKey(Key)) Key = 0;
}

//---------------------------------------------------------------------------
// R~bg̕`
//---------------------------------------------------------------------------
void __fastcall TGitViewer::CommitListBoxDrawItem(TWinControl *Control, int Index, TRect &Rect,
	TOwnerDrawState State)
{
	TListBox *lp = (TListBox*)Control;
	TCanvas  *cv = lp->Canvas;
	cv->Brush->Color = State.Contains(odSelected)? (State.Contains(odFocused)? col_selItem : col_oppItem) : lp->Color;
	cv->FillRect(Rect);

	int xp = Rect.Left + Scaled4;
	int yp = Rect.Top + Scaled2;
	bool use_fgsel = is_SelFgCol(State);

	git_rec *gp = (git_rec *)lp->Items->Objects[Index];
	if (gp) {
		//Ot
		if (!gp->graph.IsEmpty()) {
			git_rec *gp1 = (Index>0)? (git_rec *)lp->Items->Objects[Index - 1] : NULL;
			git_rec *gp2 = (Index<(lp->Count - 1))? (git_rec *)lp->Items->Objects[Index + 1] : NULL;
			UnicodeString s1 = gp1? gp1->graph : EmptyStr;
			UnicodeString s2 = gp2? gp2->graph : EmptyStr;
			TRect rc = Rect; rc.Left = xp;
			draw_GitGraph(gp->graph, s1, s2, cv, rc, gp->is_head);
		}

		xp += lp->Tag;
		if (!gp->hash.IsEmpty()) {
			//nbV
			cv->Font->Color = use_fgsel? col_fgSelItem : col_GitHash;
			cv->TextOut(xp, yp, gp->hash.SubString(1, 7));
			xp += cv->TextWidth("9999999 ");

			//
			UnicodeString tmp = FormatDateTime(TimeStampFmt, gp->f_time);
			cv->Font->Color = use_fgsel? col_fgSelItem : get_TimeColor(gp->f_time, col_fgList);
			cv->TextOut(xp, yp, tmp);
			xp += cv->TextWidth(tmp) + Scaled8;

			//^O
			if (!gp->tag.IsEmpty()) {
				TColor org_bg = cv->Brush->Color;
				int mk_h = cv->TextHeight("Q");
				int mk_w = mk_h/4;
				TPoint shape[3]  = {Point(xp, yp + mk_h/2), Point(xp + mk_w, yp), Point(xp + mk_w, yp + mk_h - 1)};
				cv->Pen->Style	 = psSolid;
				cv->Pen->Color	 = clYellow;
				cv->Brush->Color = clYellow;
				cv->Polygon(shape, 2);
				xp += (mk_w + 1);
				cv->Brush->Color = org_bg;
				out_TextEx(cv, xp, yp, gp->tag, col_bgList, col_GitTag, Scaled8);
			}

			//u`
			if (!gp->branch.IsEmpty()) {
				if (gp->is_head) out_TextEx(cv, xp, yp, _T("\u25b6"), col_GitHEAD);
				out_TextEx(cv, xp, yp, gp->branch, col_bgList, col_GitBra, Scaled8);
			}
			if (ShowRemoteAction->Checked && !gp->branch_r.IsEmpty()) {
				out_TextEx(cv, xp, yp, gp->branch_r, col_bgList, col_GitBraR, Scaled8);
			}

			//bZ[W
			cv->Font->Color = use_fgsel? col_fgSelItem : col_fgList;
			cv->TextOut(xp, yp, gp->msg);
		}
	}
	else {
		cv->Font->Color = use_fgsel? col_fgSelItem : col_Teal;
		cv->TextOut(xp, yp, lp->Items->Strings[Index]);
	}
}
//---------------------------------------------------------------------------
void __fastcall TGitViewer::CommitListBoxClick(TObject *Sender)
{
	DiffListBox->Clear();
	DiffID = EmptyStr;

	TListBox *lp = CommitListBox;
	int idx = lp->ItemIndex;
	if (idx==-1 || idx==lp->Count-1) return;

	git_rec *gp = (git_rec *)lp->Items->Objects[idx];
	if (!gp || gp->hash.IsEmpty()) return;

	DiffID = gp->hash;
	int b_idx =BranchListBox->Items->IndexOf(gp->branch);
	if (b_idx!=-1) BranchListBox->ItemIndex = b_idx;

	std::unique_ptr<TStringList> o_lst(new TStringList());
	UnicodeString prm = "diff --stat-width=120 " + gp->hash + "^ " + gp->hash;	//***
	GitBusy = true;
	if (GitShellExe(prm, RepoDir, o_lst.get()) && o_lst->Count>0) {
		TListBox *lp = DiffListBox;
		int max_fwd = 0;
		for (int i=0; i<o_lst->Count; i++) {
			UnicodeString lbuf = o_lst->Strings[i];
			if (lbuf.Pos(" | "))
				max_fwd = std::max(max_fwd, lp->Canvas->TextWidth(get_tkn(lbuf, " | ") + " "));
		}
		lp->Tag = max_fwd;
		lp->Items->Assign(o_lst.get());
		lp->ItemIndex = 0;
	}
	GitBusy = false;
	DiffScrPanel->UpdateKnob();
}
//---------------------------------------------------------------------------
void __fastcall TGitViewer::CommitListBoxKeyDown(TObject *Sender, WORD &Key, TShiftState Shift)
{
	TListBox *lp = (TListBox*)Sender;
	UnicodeString KeyStr = get_KeyStr(Key, Shift);
	UnicodeString cmd_F  = Key_to_CmdF(KeyStr);

	if		(ExeCmdListBox(lp, cmd_F) || ExeCmdListBox(lp, Key_to_CmdV(KeyStr)))
													CommitListBoxClick(NULL);
	else if (USAME_TI(cmd_F, "ReturnList"))			ModalResult = mrCancel;
	else if (is_ToLeftOpe(KeyStr, cmd_F))			BranchListBox->SetFocus();
	else if (is_ToRightOpe(KeyStr, cmd_F))			DiffListBox->SetFocus();
	else if (contained_wd_i(KeysStr_Popup, KeyStr))	show_PopupMenu(lp);
	else if (USAME_TI(get_CmdStr(cmd_F), "Pack"))	ArchiveAction->Execute();
	else return;

	if (!is_DialogKey(Key)) Key = 0;
}

//---------------------------------------------------------------------------
//T̕`
//---------------------------------------------------------------------------
void __fastcall TGitViewer::DiffListBoxDrawItem(TWinControl *Control, int Index, TRect &Rect,
	TOwnerDrawState State)
{
	TListBox *lp = (TListBox*)Control;
	TCanvas  *cv = lp->Canvas;
	cv->Brush->Color = State.Contains(odSelected)? (State.Contains(odFocused)? col_selItem : col_oppItem) : lp->Color;
	cv->FillRect(Rect);

	int xp = Rect.Left + Scaled4;
	int yp = Rect.Top + Scaled2;
	bool use_fgsel = is_SelFgCol(State);

	UnicodeString lbuf = lp->Items->Strings[Index];
	if (lbuf.Pos(" | ")) {
		//t@C
		UnicodeString s = split_tkn(lbuf, " | ");
		if (s.Pos(" => ")) {
			UnicodeString s1 = split_tkn(s, " => ");
			out_TextEx(cv, xp, yp, s1,
				use_fgsel? col_fgSelItem :
				AppFextColorAction->Checked? get_ExtColor(get_extension(s1), col_fgTxtPrv) : col_fgTxtPrv);
			out_TextEx(cv, xp, yp, " => ", use_fgsel? col_fgSelItem : col_fgTxtPrv);
		}
		out_TextEx(cv, xp, yp, s,
			use_fgsel? col_fgSelItem :
			AppFextColorAction->Checked? get_ExtColor(get_extension(Trim(s)), col_fgTxtPrv) : col_fgTxtPrv);

		//r
		TRect rc = Rect; rc.Left = xp = lp->Tag;
		RuledLnTextOut("", cv, rc, col_fgTxtPrv);
		xp = rc.Left;
		//v
		s = split_tkn_spc(lbuf) + " ";
		if (StartsText("Bin", Trim(s))) {
			out_TextEx(cv, xp, yp, s,	 use_fgsel? col_fgSelItem : AdjustColor(col_fgTxtPrv, 72));
			lbuf = ReplaceStr(lbuf, "->", "");
			out_TextEx(cv, xp, yp, lbuf, use_fgsel? col_fgSelItem : col_fgTxtPrv);
		}
		else {
			//ύX
			UnicodeString tmp = ReplaceStr(s, " ", "0");
			xp += (cv->TextWidth(tmp) - cv->TextWidth(s));
			out_TextEx(cv, xp, yp, s, use_fgsel? col_fgSelItem :col_fgTxtPrv);
			//Ot
			int g_sz = rc.Height() / 3;
			TRect g_rc = rc;
			g_rc.Left  = xp;
			g_rc.SetWidth(g_sz/1); g_rc.SetHeight(g_sz); g_rc.Offset(0, g_sz);
			for (int i=1; i<=lbuf.Length(); i++) {
				WideChar c = lbuf[i];
				cv->Brush->Color = (c=='-')? clRed : (c=='+')? clGreen : clNone;
				if (cv->Brush->Color!=clNone) {
					cv->FillRect(g_rc);
					g_rc.Offset(g_sz, 0);
				}
			}
		}
	}
	else {
		out_TextEx(cv, xp, yp, lbuf, use_fgsel? col_fgSelItem : col_fgTxtPrv);
	}
}
//---------------------------------------------------------------------------
void __fastcall TGitViewer::DiffListBoxKeyDown(TObject *Sender, WORD &Key, TShiftState Shift)
{
	TListBox *lp = (TListBox*)Sender;
	UnicodeString KeyStr = get_KeyStr(Key, Shift);
	UnicodeString cmd_F  = Key_to_CmdF(KeyStr);

	if		(ExeCmdListBox(lp, cmd_F) || ExeCmdListBox(lp, Key_to_CmdV(KeyStr)))
		;
	else if (USAME_TI(cmd_F, "ReturnList"))			ModalResult = mrCancel;
	else if (is_ToLeftOpe(KeyStr, cmd_F))			BranchListBox->SetFocus();
	else if (is_ToRightOpe(KeyStr, cmd_F))			CommitListBox->SetFocus();
	else if (contained_wd_i(KeysStr_Popup, KeyStr))	show_PopupMenu(lp);
	else return;

	if (!is_DialogKey(Key)) Key = 0;
}

//---------------------------------------------------------------------------
void __fastcall TGitViewer::GitListBoxMouseDown(TObject *Sender, TMouseButton Button,
	TShiftState Shift, int X, int Y)
{
	((TListBox *)Sender)->SetFocus();
}

//---------------------------------------------------------------------------
//`FbNAEg
//---------------------------------------------------------------------------
void __fastcall TGitViewer::ChckoutActionExecute(TObject *Sender)
{
	UnicodeString bnam = GetCurBranchName();
	if (!bnam.IsEmpty()) GitExeStr("checkout " + bnam);
}
//---------------------------------------------------------------------------
//u`̍쐬
//---------------------------------------------------------------------------
void __fastcall TGitViewer::CreBranchActionExecute(TObject *Sender)
{
	UnicodeString bnam = InputBox("u`̍쐬", "u`", EmptyStr);
	if (!bnam.IsEmpty()) GitExeStr("branch " + bnam);
}
//---------------------------------------------------------------------------
//u`̍폜
//---------------------------------------------------------------------------
void __fastcall TGitViewer::DelBranchActionExecute(TObject *Sender)
{
	UnicodeString bnam = GetCurBranchName();
	if (!bnam.IsEmpty()) {
		UnicodeString msg;
		msg.sprintf(_T("u`[%s]폜Ă悢ł?"), bnam.c_str());
		TForm *MsgDlg = CreateMessageDialog(msg, mtConfirmation,
							TMsgDlgButtons() << mbYes << mbNo << mbCancel, mbNo);
		TCheckBox *cp = new TCheckBox(MsgDlg);
		MsgDlg->ClientHeight = MsgDlg->ClientHeight + cp->Height + 12;
		cp->Caption = "Iɍ폜(&D)";
		cp->Parent	= MsgDlg;
		cp->Left	= 20;
		cp->Top		= MsgDlg->ClientHeight - cp->Height - 12;
		cp->Width	= MsgDlg->ClientWidth - cp->Left;

		if (MsgDlg->ShowModal()==mrYes) {
			UnicodeString prm;
			prm.sprintf(_T("branch -%s %s"), (cp->Checked? _T("D") : _T("d")), bnam.c_str());
			GitExeStr(prm);
		}

		delete MsgDlg;
	}
}

//---------------------------------------------------------------------------
//u`̖OύX
//---------------------------------------------------------------------------
void __fastcall TGitViewer::RenBranchActionExecute(TObject *Sender)
{
	UnicodeString bnam = GetCurBranchName();
	if (!bnam.IsEmpty()) {
		bnam = InputBox("u`̖OύX", "u`", bnam);
		if (!bnam.IsEmpty()) GitExeStr("branch -m " + bnam);
	}
}
//---------------------------------------------------------------------------
//^Ot
//---------------------------------------------------------------------------
void __fastcall TGitViewer::SetTagActionExecute(TObject *Sender)
{
	UnicodeString tnam = InputBox("^Ot", "^O", EmptyStr);
	if (!tnam.IsEmpty()) GitExeStr("tag " + tnam);
}
//---------------------------------------------------------------------------
//^O̍폜
//---------------------------------------------------------------------------
void __fastcall TGitViewer::DelTagActionExecute(TObject *Sender)
{
	git_rec *gp = GetCurCommitPtr();
	if (gp && !gp->tag.IsEmpty()) {
		UnicodeString msg;
		msg.sprintf(_T("^O[%s]폜Ă悢ł?"), gp->tag.c_str());
		if (msgbox_Sure(msg)) GitExeStr("tag -d " + gp->tag);
	}
}
//---------------------------------------------------------------------------
void __fastcall TGitViewer::DelTagActionUpdate(TObject *Sender)
{
	git_rec *gp = GetCurCommitPtr();
	((TAction *)Sender)->Visible = (gp && !gp->tag.IsEmpty());
}

//---------------------------------------------------------------------------
//}[W
//---------------------------------------------------------------------------
void __fastcall TGitViewer::MergeActionExecute(TObject *Sender)
{
	UnicodeString bnam = GetCurBranchName();
	if (!CurIsHEAD() && !bnam.IsEmpty()) {
		UnicodeString msg;
		msg.sprintf(_T("[%s]  [%s] Ƀ}[W܂?"), bnam.c_str(), RefHEAD.c_str());
		if (msgbox_Sure(msg)) GitExeStr("merge " + bnam);
	}
}

//---------------------------------------------------------------------------
//u`Rs[
//---------------------------------------------------------------------------
void __fastcall TGitViewer::CopyBranchNameActionExecute(TObject *Sender)
{
	git_rec *gp = GetCurCommitPtr();
	if (gp && !gp->branch.IsEmpty()) copy_to_Clipboard(gp->branch);
}
//---------------------------------------------------------------------------
void __fastcall TGitViewer::CopyBranchNameActionUpdate(TObject *Sender)
{
	git_rec *gp = GetCurCommitPtr();
	((TAction *)Sender)->Visible = (gp && !gp->branch.IsEmpty());
}
//---------------------------------------------------------------------------
//R~bgIDRs[
//---------------------------------------------------------------------------
void __fastcall TGitViewer::CopyCommitIDActionExecute(TObject *Sender)
{
	git_rec *gp = GetCurCommitPtr();
	if (gp && !gp->hash.IsEmpty()) copy_to_Clipboard(gp->hash);
}
//---------------------------------------------------------------------------
void __fastcall TGitViewer::CopyCommitIDActionUpdate(TObject *Sender)
{
	git_rec *gp = GetCurCommitPtr();
	((TAction *)Sender)->Visible = (gp && !gp->hash.IsEmpty());
}

//---------------------------------------------------------------------------
void __fastcall TGitViewer::ActTagActionUpdate(TObject *Sender)
{
	((TAction *)Sender)->Visible = CurIsHEAD();
}
//---------------------------------------------------------------------------
void __fastcall TGitViewer::InactBranchActionUpdate(TObject *Sender)
{
	((TAction *)Sender)->Visible = !CurIsHEAD();
}

//---------------------------------------------------------------------------
//ÕR~bgɖ߂
//---------------------------------------------------------------------------
void __fastcall TGitViewer::BrPopupMenuPopup(TObject *Sender)
{
	ResetItem->Visible = CurIsHEAD();
}
//---------------------------------------------------------------------------
void __fastcall TGitViewer::RestItemClick(TObject *Sender)
{
	int tag = ((TMenuItem *)Sender)->Tag;
	UnicodeString prm = (tag==2)? "Hard" : (tag==0)? "Soft" : "Mixed";
	UnicodeString msg;
	msg.sprintf(_T("[%s]𒼑ÕR~bgɖ߂܂(%s)?"), RefHEAD.c_str(), prm.c_str());
	if (msgbox_Sure(msg)) GitExeStr("reset --" + prm.LowerCase() + " HEAD^");
}

//---------------------------------------------------------------------------
//Diffc[N
//---------------------------------------------------------------------------
void __fastcall TGitViewer::DiffToolActionExecute(TObject *Sender)
{
	UnicodeString fnam = GetDiffFileName();
	if (!fnam.IsEmpty()) {
		UnicodeString fnam2;
		if (fnam.Pos(" => ")) {
			fnam2 = get_tkn_r(fnam, " => ");
			fnam =  get_tkn(fnam, " => ");
		}
		if (fnam2.IsEmpty()) fnam2 = fnam;

		UnicodeString prm;
		prm.sprintf(_T("difftool -y %s^:%s %s:%s"),
			DiffID.c_str(), fnam.c_str(), DiffID.c_str(), fnam2.c_str());
		GitExeStr(prm);
	}
}
//---------------------------------------------------------------------------
void __fastcall TGitViewer::DiffToolActionUpdate(TObject *Sender)
{
	((TAction *)Sender)->Enabled = !GetDiffFileName().IsEmpty();
}

//---------------------------------------------------------------------------
//A[JCu쐬
//---------------------------------------------------------------------------
void __fastcall TGitViewer::ArchiveActionExecute(TObject *Sender)
{
	git_rec *gp = GetCurCommitPtr();
	if (gp && !gp->hash.IsEmpty()) {
		UserModule->PrepareSaveDlg(_T("A[JCu̍쐬"), _T("A[JCu (*.zip)|*.zip"), _T("archive.zip"), RepoDir);
		if (UserModule->SaveDlg->Execute()) {
			UnicodeString arc_name = UserModule->SaveDlg->FileName;
			UnicodeString prm;
			prm.sprintf(_T("archive --format=zip %s --output=\"%s\""), gp->hash.c_str(), yen_to_slash(arc_name).c_str());
			GitExeStr(prm);
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TGitViewer::ArchiveActionUpdate(TObject *Sender)
{
	git_rec *gp = GetCurCommitPtr();
	((TAction *)Sender)->Visible = (gp && !gp->hash.IsEmpty());
}

//---------------------------------------------------------------------------
//̃u`\
//---------------------------------------------------------------------------
void __fastcall TGitViewer::ShowBranchesActionExecute(TObject *Sender)
{
	ShowBranchesAction->Checked = !ShowBranchesAction->Checked;
	GetCommitList();
}
//---------------------------------------------------------------------------
//[gQƂ\
//---------------------------------------------------------------------------
void __fastcall TGitViewer::ShowRemoteActionExecute(TObject *Sender)
{
	ShowRemoteAction->Checked = !ShowRemoteAction->Checked;
	GetCommitList();
}
//---------------------------------------------------------------------------
//gqʔzFKp
//---------------------------------------------------------------------------
void __fastcall TGitViewer::AppFextColorActionExecute(TObject *Sender)
{
	AppFextColorAction->Checked = !AppFextColorAction->Checked;
	DiffListBox->Repaint();
}

//---------------------------------------------------------------------------
void __fastcall TGitViewer::FormKeyDown(TObject *Sender, WORD &Key, TShiftState Shift)
{
	SpecialKeyProc(this, Key, Shift, _T(HELPTOPIC_GIT) _T("#GitViewer"));
}
//---------------------------------------------------------------------------

